/***************************************************************************
                          textview.h  -  description
                             -------------------
    begin                : tue apr 15 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    Copyright: See COPYING file that comes with this project
***************************************************************************/

#ifndef _TEXTVIEW_H_
#define _TEXTVIEW_H_

#include "view.h"
#include "syntaxhighlighter.h"

#include <qstringlist.h>
#include <qtextedit.h>
#include <qprinter.h>
#include <qlistbox.h>
#include <qaction.h>


/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa obsuguje podgld tekstowy.
 Klasa ta ma za zadanie pen obsug widoku tekstowego w trzech dostpnych
 trybach (RAWmode, RENDERmode, EDITmode). Dla kadego z nich udostpnia
 niezbdne narzdzia. Na przykad dla trybu edycji dostpne s: wyszukiwanie,
 podmiana oraz skok do linii; dla pozostaych trybw dostpne jest tylko
 wyszukiwanie. Ponad to klasa wspiera kolorowanie skadni. Domylnie jest ono
 wykrywane na podstawie rozszerzenia pliku, mona je take ustawi bezporednio
 z menu gwnego. Kolorowanie skadni jest wykonywane w trybach (RAWmode i
 EDITmode). Tryb RENDERmode jest wykorzystywany do pokazywania plikw z kodem
 HTML. Dostpna jest take moliwo zmiany rozmiaru fontu bezporednio z menu
 gwnego (wszystkie tryby). Mona rwnie wydrukowa zawarto widoku (tryby
 RAWmode i EDITmode). Poza tym klasa umoliwia take: bezporedni zmian
 rozmiaru i rodziny biecej czcionki, zmian znaku koca linii oraz rozmiaru
 tabulatora.
 */
class TextView : public View
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param parent - wskanika na rodzica,
	 @param name - nazwa dla obiektu.\n
	 Inicjowane s tutaj skadowe klasy, tworzone obiekty drukowania,
	 podgldu tekstowego i kolorowania skadni, inicjowane opcje dla menu i dla
	 paska narzdziowego, a take inicjowane kodowanie fontu.
	 */
	TextView( QWidget *parent, const char *name=0 );

	/** Destruktor klasy.\n
	 Powoduje usunicie obiektw wykorzystywanych przez biecy obiekt.
	 */
	~TextView();

	/** Metoda zwraca rodzaj biecego widoku.\n
	 @return rodzaj widoku.
	 */
	ModeOfView modeOfView() { return mModeOfView; }

	/** Resetowany jest tutaj widok oraz czyszczone bufory.\n
	 */
	void clear() {
		mTextEdit->clear();
		mTxtBuffer = ""; mMainBuffer = "";
	}

	/** Metoda przekazuje wskanika na obiekt skrtw klawiszowych dla biecego
	 podgldu oraz ustawia akcje dla menu i paska narzdziowego.\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void setKeyShortcuts( KeyShortcuts * keyShortcuts );

	/** Powoduje uaktualnienie biecego widoku podanymi danymi.\n
	 @param updateMode - tryb uaktualnienia (dostpne s: ALL, APPEND, ON_DEMAND,
	 NONE),
	 @param modeOfView - tryb podgldu (dostpne s: RAWmode, RENDERmode, HEXmode,
	 EDITmode, UNKNOWNmode),
	 @param fileName - nazwa podgldanego pliku,
	 @param binBuffer - bufor z danymi, ktrymi naley uaktualni widok.\n
	 Nazwa pliku jest niezbdna dla uzyskania rozszerzenia pliku, ktre
	 z kolei konieczne jest dla metody ustawiajcej kolorowanie skadni i formatu
	 tekstu (dla podgldu tekstowego).
	 */
	void updateContents( UpdateMode , ModeOfView, const QString &, QByteArray & binBuffer );

	/** Pobiera zawarto biecego widoku.\n
	 @param binBuffer - bufor w ktrym naley umieci dane.
	 */
	void getData( QByteArray & binBuffer );

	/** Uaktualnia informacj na pasku statusu.
	 */
	void updateStatus();

	/** Funkcja powoduje zapisanie zawartoci okna podgldu do pliku tekstowego.\n
	 @param fileName - nazwa pliku do zapisu,
	 @return TRUE poprawnie zapisano plik, FALSE - nie mona byo zapisa pliku.
	 */
	bool saveToFile( const QString & fileName );

	/** Zwraca informacje o statusie modyfikowania zawartoci widoku.\n
	 @return TRUE jeli zmodyfikowano widok
	*/
	bool isModified() const { return mTextEdit->isModified(); }

	/** Zwraca wskanik do grupy opcji menu 'Edit' dla widoku.\n
	 @return wskanik do grupy opcji menu.
	 */
	QActionGroup *editMenuBarActions() const { return mEditMenuActions; }

	/** Zwraca wskanik do grupy opcji menu 'View' dla widoku.\n
	 @return wskanik do grupy opcji menu.
	 */
	QActionGroup *viewMenuBarActions() const { return mViewMenuActions; }

	/** Zwraca wskanik do grupy opcji paska narzdziowego 'Edit' dla widoku.\n
	 @return wskanik do grupy opcji paska narzdziowego.
	 */
	QActionGroup *editToolBarActions() const { return mEditToolBarActions; }

	/** Zwraca wskanik do grupy opcji paska narzdziowego 'View' dla widoku.\n
	 @return wskanik do grupy opcji paska narzdziowego.
	 */
	QActionGroup *viewToolBarActions() const { return mViewToolBarActions; }

private:
	QWidget   *mParent;
	QPrinter  *mPrinter;
	QTextEdit *mTextEdit;

	QString mTxtBuffer;
	ModeOfView mModeOfView;

	QString mFontFamily;
	int mFontSize, mTabWidth;
	int mFontBold, mFontItalic;

	QString mInsStatStr, mCurrentLink;
	uint mRowNum, mColNum;
	bool mOvrMode;

	SyntaxHighlighter *mSyntaxHighlighter;
	SyntaxHighlighter::SyntaxHighlighting mForceKindOfSH;
	bool mAutoSH;

	enum KindOfEOL { UNKNOWNeol=0, UNIXeol, WIN_DOSeol };
	KindOfEOL mKindOfEOL;

	// find and replace dialog
	QString mFindedText, mReplacedText;
	bool mCaseSensitive, mWholeWordsOnly, mStartFindFromCursorPos, mFindForward;
	bool mPromptOnReplace;
	bool mFindOnce, mFromBegin, mBtnContinueHasPressed;
	QStringList mFindedTextList;
	QStringList mReplacedTextList;
	// 'Goto line' dialog
	QStringList mLineNumbersList;

	QString mPartTagBuf, mMainBuffer;
	Qt::TextFormat mTextFormat;

	QString mWordPrefix;
	QListBox *mCompletionListBox;

	QActionGroup *mEditActions;
	QActionGroup *mViewMenuActions, *mEditMenuActions;
	QActionGroup *mEditToolBarActions, *mViewToolBarActions;
	QActionGroup *mEolActions, *mCodingFontsActions, *mManuallyCodingActions;
	QActionGroup *mSelectHighlightingActions, *mSourceHighlightActions, *mMarkupHighlightActions;//, *mScriptsHighlightActions;
	QAction *mFindA, *mFindNextA, *mReplaceA, *mGotoLineA, *mUndoA, *mRedoA, *mCutA, *mCopyA, *mPasteA, *mPrintA;
	QAction *mZoomInA, *mZoomOutA;
	QAction *mNoneA, *mCCppA, *mHtmlA;
	QAction *mUnixEolA, *mWinDosEolA;
	QAction *mHiddenEncA, *mHiddenEncInParentA;
	QAction *mHiddenHighlSrcA, *mHiddenHighlMarA, *mHiddenHighlInParentA;
	KeyShortcuts *mKeyShortcuts;

	/** Identyfikatory akcji skrtw klawiszowych dla podgldu tekstowego.
	*/
	enum ActionsOfTextView {
		Find_TEXTVIEW=1000,
		FindNext_TEXTVIEW,
		Replace_TEXTVIEW,
		GoToLine_TEXTVIEW,
		Undo_TEXTVIEW,
		Redo_TEXTVIEW,
		Cut_TEXTVIEW,
		Copy_TEXTVIEW,
		Paste_TEXTVIEW,
		SelectAll_TEXTVIEW,
		Print_TEXTVIEW,
		ZoomIn_TEXTVIEW,
		ZoomOut_TEXTVIEW,
		TextCompletion_TEXTVIEW
	};

	/** Funkcja inicjujca biecy widok.\n
	 Wczytywane s tutaj z pliku konfiguracyjnego i ustawiane na widoku
	 nastpujce wasnoci: font, szeroko tabulatora.
	 */
	void initView();

	/** Inicjowane s tutaj opcje przeznaczone dla menu gownego oraz paska
	 narzdziowego okna gownego widoku.\n
	 Wskaniki to tych opcji (typu QActionGroup), s zwracane przez nastpujce
	 metody: @em editMenuBarActions() , @em viewMenuBarActions() ,
	 @em editToolBarActions() , @em viewToolBarActions() .
	 */
	void initMenuAndToolBarActions();

	/** Metoda powoduje zastosowanie ustawie skrtw klawiszowych dla biecego
	 podgldu.\n
	 @param keyShortcuts - wskanik na obiekt skrtw klawiszowych.
	 */
	void applyKeyShortcuts( KeyShortcuts * keyShortcuts );

	/** Wykonuje ustawienie rozmiaru tabulatora.\n
	 @param amountOfChar - ilo spacji dla tabulatora
	 */
	void setTabWidth( uint amountOfChars );

	/** Powoduje ustawienie znaku koca linii na podany w biecym dokumencie.\n
	 @param kindOfEOL - rodzaj znaku koca linii (musi by zgodny z KindOfEOL,
	 dostpne s : UNIXeol, WIN_DOSeol, UNKNOWNeol)
	 */
	void setKindOfEoL( KindOfEOL kindOfEOL );

	/** Metoda powoduje ustawienie podanego kodowania dla biecego fontu.\n
	 @param encoding - nazwa dla kodowania.
	 UWAGA: metoda nie jest jeszcze zaimplementowana.
	 */
	void setEncoding( const QString & encoding=QString::null );

	/** Wykonuje ustawienie fontu dla widoku tekstowego.\n
	 @param family - rodzina fontu
	 @param size - rozmiar fontu
	 @param bold - czy font ma by pogrubiony
	 @param italic - czy font ma by pochylony
	 */
	void setFont( const QString & family, uint size, bool bold, bool italic );

	/** Metoda powoduje uaktualnienie zaznaczenia w menu dla aktualnego
	 kolorowania.\n
	 Biece kolorowania pobierane jest z obiektu klasy 'SyntaxHighlighter', ktry
	 jest skadow tej klasy.
	 */
	void updateHighlightingAction();

	/** Zapisuje aktualnie zaznaczony (nie duszy ni 16 znakw) cig do podanej
	 zmiennej.\n
	 @param selectionsStr - tu zapisywany jest aktualnie zaznaczony tekst.
	 */
	void getCurrentSelection( QString & selectionsStr );

	/** Metoda wykorzystywana do zwikszania lub zmniejszania fontu na widoku.\n
	 @param in - warto TRUE nakazuje zwikszenie, FALSE zmniejszenie fontu,
	 @param range - mwi o ile razy zmieni rozmiar fontu (domylnie 1).
	 */
	void zoom( bool in, int range=1 );

	/** Powoduje dodanie aktualnie znalezionego cigu, jeli ju nie jest
	 zapisany, do skadowej klasy @em mFindedTextList typu QStringList.
	 */
	void updateFindedTextList();

	/** Funkcja gwna wykonujca kolejne wyszukanie.\n
	 Wywoywana jest przez slot @em slotFindNext().
	 */
	void findNext();

	/** Zwraca rozmiar dokumentu w bajtach z uwzgldnieniem znakw koca linii
	 dla systemw UNIX i DOS/Windows.
	 */
	uint weight();


	/** Tworzony jest tu obiekt listy kompletowania tekstu.\n
	 Poza tym nastpuje tutaj te poczenie niezbdnych sygnaw i slotw.
	 */
	void createListBox();

	/** Dopasowuje rozmiar listy kompletowania do iloci elementw na niej.\n
	 @param maxHeight - maksymalna wysoko,
	 @param maxWidth - maksymalna szeroko.
	 */
	void adjustListBoxSize( int maxHeight=32767, int maxWidth=32767 );

	/** Pobierana jest tutaj pozycja kursora wzgldem widoku.\n
	 @return globalna pozycja kursora na widoku.
	 */
	QPoint textCursorPoint() const;

protected:
	/** Funkcja ustawia geometri dla obiektu widoku tekstowego.\n
	 @param x - nowa pozycja X,
	 @param y - nowa pozycja Y,
	 @param w - nowa szeroko,
	 @param h - nowa wysoko.
	*/
	void setGeometry( int x, int y, int w, int h ) { mTextEdit->setGeometry( x,y, w,h); }

	/** Funkcja przechwytujca wszelkie zdarzenia dotyczce biecego obiektu.\n
	 @param o - obiekt ktrego dotyczy zdarzenie,
	 @param e - zdarzenie,
	 @return czy przekaza sterowanie dalej (TRUE), czy tu zatrzyma. \n
	 Jest ona pomocna przy przechwytywaniu zdarzenia wcinicia klawisza,
	 kliknicia oraz poruszania mysz nad obiektem widoku tekstowego.
	 */
	bool eventFilter( QObject * o, QEvent * e );

	/** Obsuga wciniecia niektrych klawiszy dla biecego widoku.\n
	 @param e - zdarzenie wcinicia klawisza,
	 @return czy przekaza sterowanie dalej (TRUE), czy tu zatrzyma.\n
	 Kombinacja 'Ctrl+Insert' - powoduje skopiowanie do schowka, natomiast
	 Ctrl+A - zaznaczea cay tekst.
	 */
	bool keyPressed( QKeyEvent * e );

	/** Funkcja obsuguje poruszanie kursorem myszy nad widokiem tekstowym.\n
	 @param e - zdarzenia dotyczce myszy.\n
	 Wykorzystywana jest w trybie 'RENDERmode' do pokazywania adresw docelowych
	 wskazywanych przez linki.
	*/
	void mouseMoveEvent( QMouseEvent * e );

	/** Funkcja obsuguje kliknicie jednym przyciskw myszy.\n
	 @param e - zdarzenia dotyczce myszy.\n
	 Wykorzystywana jest do pokazywania menu podrcznego po klikniciu prawym
	 klawiszem myszy.
	*/
	void mousePressEvent( QMouseEvent * e );

private slots:
	/** Slot wywoywany po zmianie pozycji kursora.\n
	 @param par - paragraf, w ktrym znajduje si kursor,
	 @param col - kolumna, w ktrej znajduje si kursor.
	 */
	void slotCursorPositionChanged( int, int );

	/** Kopiuje do schowka lokacje linka, nad ktrym aktualnie stoi kursor.\n
	 Cay kod tej funkcji zostal wzity ze rde Qt-3.1.1 ( QLineEdit::copy() )
	 */
	void slotCopyLinkToTheClipboard();

	/** Pusty slot wymagany dla dziaania slotu @em slotCopyLinkToTheClipboard() .
	 */
	void clipboardChanged() {}

	/** Slot powoduje zapisanie do pliku konfiguracyjnego ustawie fontu oraz
	 wielkoci tabulatora (w spacjach).
	 */
	void slotSaveSettings();


	/** Slot pokazuje dialog "Szukania cigu tekstowego". \n
	 Po wcinieciu OK wykonuje wyszukanie. Jeli cig zostanie znaleziony,
	 wtedy jest podwietlany.
	 */
	void slotFind();

	/** Slot podejmuje prb kolejnego wyszukania, ostatnio szukanego cigu.\n
	 Jeli cig zostanie znaleziony, wtedy jest podwietlany.
	 */
	void slotFindNext();

	/** Pokazuje dialog sucy do zamiany cigw.\n
	 Po wciniciu OK, jeli podany cig zostanie znaleziony, wtedy jest
	 podmieniany na podany (w dialogu).
	 */
	void slotReplace();

	/** Slot powoduje pokazanie dialogu drukowania biecego dokumentu.\n
	 Po wcinieciu OK podejmowana jest prba wydrukowania tekstu.
	 */
	void slotPrint();

	/** Slot pokazuje dialog "Skoku do wybranego numeru linii".\n
	 Po wcinieciu OK, wykonuje skok do podanej linii (dialog przechowuje rwnie
	 histori numerw linii).
	 */
	void slotGoToLine();


	/** Slot uruchamia funkcj powikszania fontu.\n
	 */
	void slotZoomIn()  { zoom( TRUE );  }

	/** Slot uruchamia funkcj zmniejszania fontu.\n
	 */
	void slotZoomOut() { zoom( FALSE ); }


	/** Slot powoduje uruchomienie funkcji zmieniajcej znak koca linii.\n
	 @param action - wybrana w menu opcja.\n
	 Slot przeznaczony jest do wykorzystania przez 'connect' dla obiektu 'QAction'
	 */
	void slotChangeEOL( QAction * action );

	/** Slot powoduje uruchomienie funkcji zmieniajcej kodowanie biecego fontu.\n
	 @param action - wybrana w menu opcja.\n
	 Ponadto wyczane jest tu zaznaczenie poprzednio aktywnej opcji w menu.
	 Slot przeznaczony jest do wykorzystania przez 'connect' dla obiektu 'QAction'.
	 */
	void slotChangeEncoding( QAction * action );

	/** Slot powoduje uruchomienie funkcji zmieniajcej podwietlanie skdni.\n
	 @param action - wybrana w menu opcja.\n
	 Ponadto wyczane jest tu zaznaczenie poprzednio aktywnej opcji w menu.
	 Slot przeznaczony jest do wykorzystania przez 'connect' dla obiektu 'QAction'.
	*/
	void slotChangeHighlighting( QAction * action );


	/** Metoda powoduje pokazanie list z pasujcymi cigami do wpisanego
	 fragmentu.\n
	 Zbierane s tu na liste pasujce cigi tekstowe z caego tekstu, jeli
	 znaleziono pasujce cigu wtedy wywietlana jest ich lista.
	 */
	void slotCompleteText();

	/** Slot wywoywany po wybraniu pasujcego cigu z listy kompletowania.\n
	 @param item - wskanik na wybrany element listy.
	 */
	void slotItemChosen( QListBoxItem *item=0 );

public slots:
	/** Slot powoduje zastosownie ustawie z przekazanych z dialogu ustawie.\n
	 @param tvs - struktura opisujca wszystkie waciwoci dla widoku tekstowego
	 na dialogu ustawie.
	 */
	void slotApplyTextViewSettings( const TextViewSettings & tvs );

};

#endif
